% GMRESPolyMain.m
% Bernd Fischer: 10.03.04
%
% -- calculates eigenvalues of either Galerkin or 
%    SUPG matrix for wind (0,1)
%
% -- assembles the SUPG matrix for arbitrary wind
%
% -- applies GMRES, BiCGstab to the generated matrix with
%
% change epsilon, Delta, n as required
%

  clear

%%Fsup parameter
   n=8;            %Gridsize:  h=1/n;
   theta=0;          %wind direction: beta_x=-sin(theta); beta_y=cos(theta); 
   epsilon=0.01;      %viscosity parameter
   delta=0.4;
  
   
%%GMRES, BiCGstab parameter  
   N=(n-1)*(n-1); rand('seed',0);  
   
   b    =zeros(N,1); %right hand side
   x0   =rand(N,1);  %initial guess
   
   itmax=101;         %upper bound on number of iterations
   tol  =1e-10;      %tolerance for stopping criterion |rk|/b <= tol   
   
%%Ritzvalues
   m=15;   %maximal number
   
%%%%%%%%%%

   h=1/n; nx=n-1; ny=nx; 
   beta_x=-sin(theta); beta_y=cos(theta); 
   
      
   %%compute eigenvalues of Fsup
	   [EigFsup,VFsup,lambda,mu,gamma]=...
          eigFsup(h,nx,epsilon,delta,beta_x,beta_y);
	  
	   
   %%plot eigenvalues
      figure(1)   
      plot(real(EigFsup),imag(EigFsup),'r.','Markersize',20)
      axis([0,0.05,-.2,.2])
      title('Eigenvalues of Fsup')
      
   
  %%generate Fsup
         [Fsup]=generate_Fsup(h,nx,epsilon,delta,beta_x,beta_y);
         
         %Fsup=diag(eigFsup);
		 
   %%apply GMRES to Fsup (without preconditioning)	
        
         [x,resGMRES,Ritz] = GmresRitz(Fsup,b,x0,itmax,tol,'pmethod');
         [x, resBiCGstab]=bicgstab(Fsup,b,x0,itmax,tol,'pmethod');
	  
	     figure(2)
         semilogy(0:length(resGMRES)-1,resGMRES,'r','linewidth',2);
         hold on
         semilogy(0:length(resBiCGstab)-1,resBiCGstab,'g','linewidth',2);
         hold off
	     xlabel('iterations')
         title('GMRES (r), BiCGstab (g)')
         
   %%plot Ritzvalues
   polynom=menu('Ritzvalues?', 'Yes', 'No');

if polynom == 1 
   figure(1)   
   for j=1:m
       plot(real(EigFsup),imag(EigFsup),'g.','Markersize',20)
       hold on
       plot(real(Ritz(j,1:j)),imag(Ritz(j,1:j)),'r.','Markersize',20)
       drawnow
       title('Eigenvalues and Ritz values (GMRES)')
       xlabel(['Polynomial degree: ', int2str(j)])
       %axis([0,0.05,-.2,.2])
       choice=menu('Weiter?','    Ja    ','    Nein    ');
       if choice ~= 1, break, end
   clf, 
   end
end
return
