    function [xMR, plotresMR, ploterrMR, xGAL, plotresGAL, ploterrGAL] = ...
        MINRES_Poly(A, f, x0, n_max, tol, matvec, x_star)
%   BF 10.09.04
%
%   implementation of the minimal residual algorithm
%   with polynomial coefficients

    %%  initialize
    k               =   1;
    N               =   length(x0);
    q               =   zeros(N, 1);
    q_tilde         =   f - feval( matvec, A, x0 );
    beta            =   norm(q_tilde);
    c_old           =   1;
    c               =   1;
    s_old           =   0;
    s               =   0;
    wMR             =   zeros(N, 1);
    wMR_old         =   zeros(N, 1);
    eta_hat         =   beta;
    xMR             =   x0;
    norm_r          =   beta;
    norm_r0         =   beta;
    norm_e0         =   norm( x_star - x0 );

    %%  only in the long version
    plotresMR       =   zeros(n_max, 1);
    plotresMR(k)    =   log10( norm_r / norm_r0 );
    ploterrMR       =   zeros(n_max, 1);
    ploterrMR(k)    =   log10( norm( x_star - xMR ) / norm_e0 );
    plotresGAL      =   zeros(n_max, 1);
    plotresGAL(k)   =   log10( norm_r / norm_r0 );
    ploterrGAL      =   zeros(n_max, 1);
    ploterrGAL(k)   =   log10( norm( x_star - xMR ) / norm_e0 );
   
    while ( k < n_max+1 ) & ( norm_r / norm_r0 > tol )

        k               =   k + 1;

        %%  Lanczos
        q_old           =   q;
        q               =   q_tilde / beta;
        Aq              =   feval( matvec, A, q );
        Gamma(k-1)      =   beta;
        alpha           =   q' * Aq;
        q_tilde         =   Aq - alpha * q - beta * q_old;
        beta_old        =   beta;
        beta            =   norm(q_tilde);

        %%  QR factorization
        c_oold          =   c_old;
        c_old           =   c;
        s_oold          =   s_old;
        s_old           =   s;

        r1_hat          =   c_old * alpha - c_oold * s_old * beta_old;
        r1              =   sqrt( r1_hat^2 + beta^2 );
        r2              =   s_old * alpha + c_oold * c_old * beta_old;
        r3              =   s_oold * beta_old;

        %%  Givens rotation
        c               =   r1_hat / r1;
        s               =   beta / r1;

        %%  update
        wMR_oold        =   wMR_old;
        wMR_old         =   wMR;
        wMR             =   ( q - r3 * wMR_oold - r2 * wMR_old ) / r1;
        xMR             =   xMR + c * eta_hat * wMR;
        %norm_rMR        =   norm_rMR * s;

        if abs(c) > eps,
            xGAL        =   xMR + s * s * eta_hat * wMR / c;
        else
             disp([int2str(k),' no Galerkin iterate exists']);
        end;

        eta_hat         =   - s * eta_hat;

        %%  only in the long version
        norm_r          =   norm( f - feval( matvec, A, xMR ) );
        plotresMR(k)    =   log10( norm_r / norm_r0 );
        ploterrMR(k)    =   log10( norm( x_star - xMR ) / norm_e0 );

        norm_rGAL       =   norm( f - feval( matvec, A, xGAL ) );
        plotresGAL(k)   =   log10( norm_rGAL / norm_r0 );
        ploterrGAL(k)   =   log10( norm( x_star - xGAL ) / norm_e0 );

        if rem(k, 5) == 0,
            disp([k, norm_r / norm_r0, norm_rGAL / norm_r0]);
        end;

    end;    %   while

    %%  only in the long version
    plotresMR       =   plotresMR(1:k);
    plotresGAL      =   plotresGAL(1:k);
    ploterrMR       =   ploterrMR(1:k);
    ploterrGAL      =   ploterrGAL(1:k);

return;
